"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const serve_1 = require("@ionic/cli-utils/lib/serve");
const utils_1 = require("@ionic/cli-utils/lib/integrations/cordova/utils");
const base_1 = require("./base");
class RunCommand extends base_1.CordovaCommand {
    getRunner() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.serveRunner) {
                this.serveRunner = yield serve_1.ServeRunner.createFromProjectType(this.env, this.env.project.type);
            }
            return this.serveRunner;
        });
    }
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const options = [
                {
                    name: 'list',
                    description: 'List all available Cordova targets',
                    type: Boolean,
                    groups: [cli_utils_1.OptionGroup.Cordova],
                },
                // Build Options
                {
                    name: 'build',
                    description: 'Do not invoke Ionic build/serve',
                    type: Boolean,
                    default: true,
                },
                ...serve_1.COMMON_SERVE_COMMAND_OPTIONS,
                // Cordova Options
                {
                    name: 'debug',
                    description: 'Mark as a debug build',
                    type: Boolean,
                    groups: [cli_utils_1.OptionGroup.Cordova],
                    hint: 'cordova',
                },
                {
                    name: 'release',
                    description: 'Mark as a release build',
                    type: Boolean,
                    groups: [cli_utils_1.OptionGroup.Cordova],
                    hint: 'cordova',
                },
                {
                    name: 'device',
                    description: 'Deploy build to a device',
                    type: Boolean,
                    groups: [cli_utils_1.OptionGroup.Cordova],
                    hint: 'cordova',
                },
                {
                    name: 'emulator',
                    description: 'Deploy build to an emulator',
                    type: Boolean,
                    groups: [cli_utils_1.OptionGroup.Cordova],
                    hint: 'cordova',
                },
                {
                    name: 'cordova-target',
                    description: `Deploy build to a device (use ${chalk_1.default.green('--list')} to see all)`,
                    type: String,
                    groups: [cli_utils_1.OptionGroup.Advanced, cli_utils_1.OptionGroup.Cordova],
                    hint: 'cordova',
                },
                {
                    name: 'buildConfig',
                    description: 'Use the specified build configuration',
                    groups: [cli_utils_1.OptionGroup.Advanced, cli_utils_1.OptionGroup.Cordova],
                    hint: 'cordova',
                },
            ];
            try {
                const runner = yield this.getRunner();
                const libmetadata = yield runner.getCommandMetadata();
                options.push(...libmetadata.options || []);
            }
            catch (e) {
                if (!(e instanceof errors_1.RunnerNotFoundException)) {
                    throw e;
                }
            }
            return {
                name: 'run',
                type: 'project',
                description: 'Run an Ionic project on a connected device',
                longDescription: `
Like running ${chalk_1.default.green('cordova run')} or ${chalk_1.default.green('cordova emulate')} directly, but also uses the dev server from ${chalk_1.default.green('ionic serve')} for livereload functionality.

For Android and iOS, you can setup Remote Debugging on your device with browser development tools using these docs${chalk_1.default.cyan('[1]')}.

Just like with ${chalk_1.default.green('ionic cordova build')}, you can pass additional options to the Cordova CLI using the ${chalk_1.default.green('--')} separator.

${chalk_1.default.cyan('[1]')}: ${chalk_1.default.bold('https://ionicframework.com/docs/developer-resources/developer-tips/')}
      `,
                exampleCommands: base_1.CORDOVA_BUILD_EXAMPLE_COMMANDS,
                inputs: [
                    {
                        name: 'platform',
                        description: `The platform to run (e.g. ${['android', 'ios'].map(v => chalk_1.default.green(v)).join(', ')})`,
                    },
                ],
                options,
            };
        });
    }
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.preRunChecks();
            if (options['noproxy']) {
                this.env.log.warn(`The ${chalk_1.default.green('--noproxy')} option has been deprecated. Please use ${chalk_1.default.green('--no-proxy')}.`);
                options['proxy'] = false;
            }
            if (options['x']) {
                options['proxy'] = false;
            }
            if (!options['build'] && options['livereload']) {
                this.env.log.warn(`No livereload with ${chalk_1.default.green('--no-build')}.`);
                options['livereload'] = false;
            }
            const metadata = yield this.getMetadata();
            if (options['list']) {
                if (!options['device'] && !options['emulator']) {
                    if (metadata.name === 'emulate') {
                        options['emulator'] = true;
                    }
                }
                const args = utils_1.filterArgumentsForCordova(metadata, options);
                yield this.runCordova(['run', ...args.slice(1)], {});
                throw new errors_1.FatalException('', 0);
            }
            if (!inputs[0]) {
                const platform = yield this.env.prompt({
                    type: 'input',
                    name: 'platform',
                    message: `What platform would you like to run (${['android', 'ios'].map(v => chalk_1.default.green(v)).join(', ')}):`,
                });
                inputs[0] = platform.trim();
            }
            yield this.checkForPlatformInstallation(inputs[0]);
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ConfigXml } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/integrations/cordova/config'));
            const { registerShutdownFunction } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/process'));
            const conf = yield ConfigXml.load(this.env.project.directory);
            registerShutdownFunction(() => {
                conf.resetContentSrc();
                conf.saveSync();
            });
            const metadata = yield this.getMetadata();
            if (options['livereload']) {
                const { serve } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/serve'));
                const details = yield serve(this.env, inputs, utils_1.generateBuildOptions(metadata, inputs, options));
                if (details.externallyAccessible === false) {
                    const extra = serve_1.LOCAL_ADDRESSES.includes(details.externalAddress) ? '\nEnsure you have proper port forwarding setup from your device to your computer.' : '';
                    this.env.log.warn(`Your device or emulator may not be able to access ${chalk_1.default.bold(details.externalAddress)}.${extra}\n\n`);
                }
                conf.writeContentSrc(`${details.protocol || 'http'}://${details.externalAddress}:${details.port}`);
                yield conf.save();
            }
            else {
                if (options.build) {
                    const { build } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/build'));
                    yield build(this.env, inputs, utils_1.generateBuildOptions(metadata, inputs, options));
                }
            }
            yield this.runCordova(utils_1.filterArgumentsForCordova(metadata, options), { logOptions: { prefix: chalk_1.default.dim('[cordova]'), wrap: false } });
        });
    }
}
exports.RunCommand = RunCommand;
