"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const cli_framework_1 = require("@ionic/cli-framework");
const format_1 = require("@ionic/cli-framework/utils/format");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const cli_utils_1 = require("@ionic/cli-utils");
const base_1 = require("./base");
const SSH_KEY_TYPES = ['ecdsa', 'ed25519', 'rsa'];
class SSHGenerateCommand extends base_1.SSHBaseCommand {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'generate',
                type: 'global',
                description: 'Generates a private and public SSH key pair',
                inputs: [
                    {
                        name: 'key-path',
                        description: 'Destination of private key file',
                    },
                ],
                options: [
                    {
                        name: 'type',
                        description: `The type of key to generate: ${SSH_KEY_TYPES.map(v => chalk_1.default.green(v)).join(', ')}`,
                        default: 'rsa',
                        aliases: ['t'],
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'bits',
                        description: 'Number of bits in the key',
                        aliases: ['b'],
                        default: '2048',
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'annotation',
                        description: 'Annotation (comment) in public key. Your Ionic email address will be used',
                        aliases: ['C'],
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                ],
            };
        });
    }
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.checkForOpenSSH();
            const config = yield this.env.config.load();
            yield this.env.session.getUserToken();
            if (!options['annotation']) {
                options['annotation'] = config.user.email;
            }
            cli_framework_1.validate(String(options['type']), 'type', [cli_framework_1.contains(SSH_KEY_TYPES, { caseSensitive: false })]);
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { getGeneratedPrivateKeyPath } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/ssh'));
            const { bits, annotation } = options;
            const config = yield this.env.config.load();
            const keyPath = inputs[0] ? format_1.expandPath(String(inputs[0])) : yield getGeneratedPrivateKeyPath(config.user.id);
            const keyPathDir = path.dirname(keyPath);
            const pubkeyPath = `${keyPath}.pub`;
            if (!(yield fs_1.pathExists(keyPathDir))) {
                yield fs_1.fsMkdirp(keyPathDir, 0o700);
                this.env.log.msg(`Created ${chalk_1.default.bold(format_1.prettyPath(keyPathDir))} directory for you.\n`);
            }
            if (yield fs_1.pathExists(keyPath)) {
                const confirm = yield this.env.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: `Key ${chalk_1.default.bold(format_1.prettyPath(keyPath))} exists. Overwrite?`,
                });
                if (confirm) {
                    yield fs_1.fsUnlink(keyPath);
                }
                else {
                    this.env.log.msg(`Not overwriting ${chalk_1.default.bold(format_1.prettyPath(keyPath))}.`);
                    return;
                }
            }
            this.env.log.info('Enter a passphrase for your private key.\n' +
                `You will be prompted to provide a ${chalk_1.default.bold('passphrase')}, which is used to protect your private key should you lose it. (If someone has your private key, they can impersonate you!) Passphrases are recommended, but not required.\n`);
            this.env.close();
            const shellOptions = { stdio: 'inherit', showCommand: false, showError: false };
            yield this.env.shell.run('ssh-keygen', ['-q', '-t', String(options['type']), '-b', String(bits), '-C', String(annotation), '-f', keyPath], shellOptions);
            this.env.open();
            this.env.log.nl();
            this.env.log.ok('A new pair of SSH keys has been generated!\n' +
                `Private Key (${chalk_1.default.bold(format_1.prettyPath(keyPath))}): Keep this safe!\n` +
                `Public Key (${chalk_1.default.bold(format_1.prettyPath(pubkeyPath))}): Give this to all your friends!\n`);
            this.env.log.msg(`${chalk_1.default.bold('Next steps:')}\n` +
                ` * Add your public key to Ionic: ${chalk_1.default.green('ionic ssh add ' + format_1.prettyPath(pubkeyPath))}\n` +
                ` * Use your private key for secure communication with Ionic: ${chalk_1.default.green('ionic ssh use ' + format_1.prettyPath(keyPath))}`);
        });
    }
}
exports.SSHGenerateCommand = SSHGenerateCommand;
